// A program to display a textured brickwall
//
// Use 'c' to render a simple coloured surface 
// Use 't' to render a textured surface

#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glut.h>
#include <fstream>
using namespace std;

#include "Trackball.h"
#include "Utilities.h"
#include "GroundWithGrass.h"

const int windowWidth=400;
const int windowHeight=400;

CTrackball trackball;
CGroundWithGrass ground;

// texture parameters
GLuint texName;
GLubyte *texture;
int textureWidth, textureHeight;

typedef enum {COLOURED, TEXTURED} RENDER_OPTION;
RENDER_OPTION current_render_option=TEXTURED;


void handleMouseMotion(int x, int y)
{	
	trackball.tbMotion(x, y);
}

void handleMouseClick(int button, int state, int x, int y)
{
	trackball.tbMouse(button, state, x, y);
}

void handleKeyboardEvent(unsigned char key, int x, int y)
{
	switch (key)
	{
		case 'c': current_render_option=COLOURED; 
					glutPostRedisplay(); break;
		case 't': current_render_option=TEXTURED; 
					glutPostRedisplay(); break;
		default: trackball.tbKeyboard(key);
	}
}

void display(void)
{
	glMatrixMode( GL_MODELVIEW );	// Set the view matrix ...
	glLoadIdentity();				// ... to identity.
    gluLookAt(0,0,6, 0,0,0, 0,1,0); // camera is on the z-axis
	trackball.tbMatrix();			// rotate scene using the trackball

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	ground.draw();

	glTranslatef(-1,0,0);
	if (current_render_option==TEXTURED)
	{
		glBindTexture(GL_TEXTURE_2D, texName);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

		glEnable(GL_TEXTURE_2D);
		glBegin(GL_QUADS);	// Back
		glTexCoord2f(0,0);
		glVertex3f(0,0,0);
		glTexCoord2f(2,0);
		glVertex3f(2,0,0);
		glTexCoord2f(2,1);
		glVertex3f(2,1,0);
		glTexCoord2f(0,1);
		glVertex3f(0,1,0);
		glEnd();
		glBegin(GL_QUADS);	// Front
		glTexCoord2f(0,0);
		glVertex3f(0,0,0.2);
		glTexCoord2f(2,0);
		glVertex3f(2,0,0.2);
		glTexCoord2f(2,1);
		glVertex3f(2,1,0.2);
		glTexCoord2f(0,1);
		glVertex3f(0,1,0.2);
		glEnd();
		glBegin(GL_QUADS);	// Top
		glTexCoord2f(0,1);
		glVertex3f(0,1,0.2);
		glTexCoord2f(2,1);
		glVertex3f(2,1,0.2);
		glTexCoord2f(2,0.9);
		glVertex3f(2,1,0.0);
		glTexCoord2f(0,0.9);
		glVertex3f(0,1,0.0);
		glEnd();
		glBegin(GL_QUADS);	// Right
		glTexCoord2f(1,0);
		glVertex3f(2,0,0.2);
		glTexCoord2f(0.9,0);
		glVertex3f(2,0,0);
		glTexCoord2f(0.9,1);
		glVertex3f(2,1,0);
		glTexCoord2f(1,1);
		glVertex3f(2,1,0.2);
		glEnd();
		glBegin(GL_QUADS);	// Left
		glTexCoord2f(0,0);
		glVertex3f(0,0,0.2);
		glTexCoord2f(0.1,0);
		glVertex3f(0,0,0);
		glTexCoord2f(0.1,1);
		glVertex3f(0,1,0);
		glTexCoord2f(0,1);
		glVertex3f(0,1,0.2);
		glEnd();
		glDisable(GL_TEXTURE_2D);
	}
	else if (current_render_option==COLOURED)
	{	
		glBegin(GL_QUADS);	// Back
		glColor3f(0,1,0);
		glVertex3f(0,0,0);
		glVertex3f(2,0,0);
		glVertex3f(2,1,0);
		glVertex3f(0,1,0);
		glEnd();
		glBegin(GL_QUADS);	// Front
		glColor3f(0,0,1);
		glVertex3f(0,0,0.2);
		glVertex3f(2,0,0.2);
		glVertex3f(2,1,0.2);
		glVertex3f(0,1,0.2);
		glEnd();
		glBegin(GL_QUADS);	// Top
		glColor3f(1,0,0);
		glVertex3f(0,1,0.2);
		glVertex3f(2,1,0.2);
		glVertex3f(2,1,0.0);
		glVertex3f(0,1,0.0);
		glEnd();
		glBegin(GL_QUADS);	// Right
		glColor3f(1,0,1);
		glVertex3f(2,0,0.2);
		glVertex3f(2,0,0);
		glVertex3f(2,1,0);
		glVertex3f(2,1,0.2);
		glEnd();
		glBegin(GL_QUADS);	// Left
		glColor3f(0,1,1);
		glVertex3f(0,0,0.2);
		glVertex3f(0,0,0);
		glVertex3f(0,1,0);
		glVertex3f(0,1,0.2);
		glEnd();
	}

	glFlush();
	glutSwapBuffers();
}

void init(void) 
{
	// select clearing color (for glClear)
	glClearColor (1,1,1,1);	// RGB-value for white
	// enable depth buffering
	glEnable(GL_DEPTH_TEST);
	// initialize view (simple orthographic projection)
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(33,1,2,10);
	trackball.tbInit(GLUT_LEFT_BUTTON);

	ground.init();

	// load texture
	ifstream textureFile;
//	char* fileName="Sulley3.ppm";
	char* fileName="brokenbrick.ppm";
	textureFile.open(fileName, ios::in);
	if (textureFile.fail())
	{ 
		cout << "\n Error loading the texture"; cout.flush();
		exit(0);
	}
	skipLine(textureFile);
	skipLine(textureFile);
	textureFile >> textureWidth;
	textureFile >> textureHeight;
	int maxRGBValue;
	textureFile >> maxRGBValue;
	texture = new GLubyte[textureWidth*textureHeight*4];

	int k,l,c;	
	for(k=textureHeight-1;k>=0;k--)
		for(l=0;l<textureWidth;l++)
		{
			textureFile >> c;
			texture[(k*textureWidth+l)*4]=(GLubyte) c;
			textureFile >> c;
			texture[(k*textureWidth+l)*4+1]=(GLubyte) c;
			textureFile >> c;
			texture[(k*textureWidth+l)*4+2]=(GLubyte) c;
			texture[(k*textureWidth+l)*4+3]=(GLubyte) 255;
		}

	// initialise wall texture
	glGenTextures(1, &texName);
	glBindTexture(GL_TEXTURE_2D, texName);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, textureWidth, textureHeight, 
				0, GL_RGBA, GL_UNSIGNED_BYTE, texture);	
}

void reshape(int width, int height ) {
	// Called at start, and whenever user resizes component
	int size = min(width, height);
	glViewport(0, 0, size, size);  // Largest possible square
	trackball.tbReshape(width, height);
}


// create a double buffered colour window
int main(int argc, char** argv)
{
	glutInit(&argc, argv);		
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(windowWidth, windowHeight); 
	glutInitWindowPosition(100, 100);
	glutCreateWindow("Texture Mapping");
	init ();								// initialise view
	glutMouseFunc(handleMouseClick);		// Set function to handle mouse clicks
	glutMotionFunc(handleMouseMotion);		// Set function to handle mouse motion
	glutKeyboardFunc(handleKeyboardEvent);	// Set function to handle keyboard input
	glutDisplayFunc(display);		// Set function to draw scene
	glutReshapeFunc(reshape);		// Set function called if window gets resized
	glutMainLoop();
	return 0;
}
